@@ -1,80 +1,81 @@
 (*
 QCheck: Random testing for OCaml
-copyright (c) 2013-2017, Guillaume Bury, Simon Cruanes, Vincent Hugot, Jan Midtgaard
+copyright (c) 2013-2017, Guillaume Bury, Simon Cruanes, Vincent Hugot,
+Jan Midtgaard, Julien Debon, Valentin Chaboche
 all rights reserved.
 *)
 
 (** {1 Quickcheck inspired property-based testing} *)
 
 (** The library takes inspiration from Haskell's QuickCheck library. The
-rough idea is that the programmer describes invariants that values of
-a certain type need to satisfy ("properties"), as functions from this type
-to bool. She also needs to describe how to generate random values of the type,
-so that the property is tried and checked on a number of random instances.
+    rough idea is that the programmer describes invariants that values of
+    a certain type need to satisfy ("properties"), as functions from this type
+    to bool. She also needs to describe how to generate random values of the type,
+    so that the property is tried and checked on a number of random instances.
 
-This explains the organization of this module:
+    This explains the organization of this module:
 
-- {! 'a arbitrary} is used to describe how to generate random values,
-  shrink them (make counter-examples as small as possible), print
-  them, etc. Auxiliary modules such as {!Gen}, {!Print}, and {!Shrink}
-  can be used along with {!make} to build one's own arbitrary instances.
+    - {! 'a arbitrary} is used to describe how to generate random values,
+      shrink them (make counter-examples as small as possible), print
+      them, etc. Auxiliary modules such as {!Gen}, {!Print}, and {!Shrink}
+      can be used along with {!make} to build one's own arbitrary instances.
 
-- {!Test} is used to describe a single test, that is, a property of
-  type ['a -> bool] combined with an ['a arbitrary] that is used to generate
-  the test cases for this property. Optional parameters
-  allow to specify the random generator state, number of instances to generate
-  and test, etc.
+    - {!Test} is used to describe a single test, that is, a property of
+      type ['a -> bool] combined with an ['a arbitrary] that is used to generate
+      the test cases for this property. Optional parameters
+      allow to specify the random generator state, number of instances to generate
+      and test, etc.
 
 
-Examples:
+    Examples:
 
-  - List.rev is involutive:
+    - List.rev is involutive:
 
-{[
+    {[
 
-let test =
-  QCheck.(Test.make ~count:1000
-   (list int) (fun l -> List.rev (List.rev l) = l));;
+      let test =
+        QCheck.(Test.make ~count:1000
+                  (list int) (fun l -> List.rev (List.rev l) = l));;
 
-QCheck.Test.check_exn test;;
-]}
+      QCheck.Test.check_exn test;;
+    ]}
 
-  - Not all lists are sorted (false property that will fail. The 15 smallest
-    counter-example lists will be printed):
+    - Not all lists are sorted (false property that will fail. The 15 smallest
+      counter-example lists will be printed):
 
-{[
-let test = QCheck.(
-  Test.make
-    ~count:10_000 ~max_fail:3
-    (list small_nat)
-    (fun l -> l = List.sort compare l));;
-QCheck.Test.check_exn test;;
-]}
+    {[
+      let test = QCheck.(
+          Test.make
+            ~count:10_000 ~max_fail:3
+            (list small_nat)
+            (fun l -> l = List.sort compare l));;
+      QCheck.Test.check_exn test;;
+    ]}
 
 
-  - generate 20 random trees using {! Gen.fix} :
+    - generate 20 random trees using {! Gen.fix} :
 
-{[
-type tree = Leaf of int | Node of tree * tree
+    {[
+      type tree = Leaf of int | Node of tree * tree
 
-let leaf x = Leaf x
-let node x y = Node (x,y)
+      let leaf x = Leaf x
+      let node x y = Node (x,y)
 
-let g = QCheck.Gen.(sized @@ fix
-  (fun self n -> match n with
-    | 0 -> map leaf nat
-    | n ->
-      frequency
-        [1, map leaf nat;
-         2, map2 node (self (n/2)) (self (n/2))]
-    ))
+      let g = QCheck.Gen.(sized @@ fix
+                            (fun self n -> match n with
+                               | 0 -> map leaf nat
+                               | n ->
+                                 frequency
+                                   [1, map leaf nat;
+                                    2, map2 node (self (n/2)) (self (n/2))]
+                            ))
 
-Gen.generate ~n:20 g;;
-]}
+          Gen.generate ~n:20 g;;
+    ]}
 
-More complex and powerful combinators can be found in Gabriel Scherer's
-{!Generator} module. Its documentation can be found
-{{:http://gasche.github.io/random-generator/doc/Generator.html } here}.
+    More complex and powerful combinators can be found in Gabriel Scherer's
+    {!Generator} module. Its documentation can be found
+    {{:http://gasche.github.io/random-generator/doc/Generator.html } here}.
 *)
 
 val (==>) : bool -> bool -> bool
@@ -100,8 +101,8 @@ val assume : bool -> unit
     Example:
     {[
       Test.make (list int) (fun l ->
-        assume (l <> []);
-        List.hd l :: List.tl l = l)
+          assume (l <> []);
+          List.hd l :: List.tl l = l)
     ]}
 
     @since 0.5.1
@@ -115,8 +116,8 @@ val assume_fail : unit -> 'a
     Example:
     {[
       Test.make (list int) (function
-        | [] -> assume_fail ()
-        | _::_ as l -> List.hd l :: List.tl l = l)
+          | [] -> assume_fail ()
+          | _::_ as l -> List.hd l :: List.tl l = l)
     ]}
 
     @since 0.5.1
@@ -168,13 +169,16 @@ module Gen : sig
       @since 0.13 *)
 
   val oneof : 'a t list -> 'a t
-  (** Constructs a generator that selects among a given list of generators. *)
+  (** Constructs a generator that selects among a given list of generators.
+      @raise Invalid_argument or Failure if list is empty *)
 
   val oneofl : 'a list -> 'a t
-  (** Constructs a generator that selects among a given list of values. *)
+  (** Constructs a generator that selects among a given list of values.
+      @raise Invalid_argument or Failure if list is empty *)
 
   val oneofa : 'a array -> 'a t
-  (** Constructs a generator that selects among a given array of values. *)
+  (** Constructs a generator that selects among a given array of values.
+      @raise Invalid_argument or Failure if list is empty *)
 
   val frequency : (int * 'a t) list -> 'a t
   (** Constructs a generator that selects among a given list of generators.
@@ -208,6 +212,30 @@ module Gen : sig
       @since 0.11
   *)
 
+  val range_subset : size:int -> int -> int -> int array t
+  (** [range_subset ~size:k low high] generates an array of length [k]
+      of sorted distinct integers in the range [low..high] (included).
+
+      Complexity O(k log k), drawing [k] random integers.
+
+      @raise Invalid_argument outside the valid region [0 <= k <= high-low+1].
+
+      @since 0.18
+  *)
+
+  val array_subset : int -> 'a array -> 'a array t
+  (** [array_subset k arr] generates a sub-array of [k] elements
+      at distinct positions in the input array [arr],
+      in the same order.
+
+      Complexity O(k log k), drawing [k] random integers.
+
+      @raise Invalid_argument outside the valid region
+      [0 <= size <= Array.length arr].
+
+      @since 0.18
+  *)
+
   val unit : unit t (** The unit generator. *)
 
   val bool : bool t (** The boolean generator. *)
@@ -310,7 +338,12 @@ module Gen : sig
   val array_repeat : int -> 'a t -> 'a array t
   (** [array_repeat i g] builds an array generator from exactly [i] elements generated by [g]. *)
 
-  val opt : 'a t -> 'a option t (** An option generator. *)
+  val opt : ?ratio:float -> 'a t -> 'a option t
+  (**  An option generator, with optional ratio.
+      @param ratio a float between [0.] and [1.] indicating the probability of a sample to be [Some _]
+      rather than [None].
+      @since 0.18 ([?ratio] parameter)
+  *)
 
   val pair : 'a t -> 'b t -> ('a * 'b) t (** Generates pairs. *)
 
@@ -347,8 +380,14 @@ module Gen : sig
       @since 0.11 *)
 
   val string_readable : string t
-  (** Builds a string generator using the {!char} character generator.
-      @since 0.11 *)
+  (** Builds a string generator using the {!printable} character generator.
+      @since 0.11
+      @deprecated use {!string_printable} *)
+  [@@deprecated "see string_printable"]
+
+  val string_printable : string t
+  (** Builds a string generator using the {!printable} character generator.
+      @since 0.18 *)
 
   val small_string : ?gen:char t -> string t
   (** Builds a string generator, length is {!small_nat}
@@ -403,33 +442,89 @@ module Gen : sig
       The passed size-parameter should decrease to ensure termination. *)
 
   (** Example:
-  {[
-  type tree = Leaf of int | Node of tree * tree
+      {[
+        type tree = Leaf of int | Node of tree * tree
 
-  let leaf x = Leaf x
-  let node x y = Node (x,y)
+        let leaf x = Leaf x
+        let node x y = Node (x,y)
 
-  let g = QCheck.Gen.(sized @@ fix
-    (fun self n -> match n with
-      | 0 -> map leaf nat
-      | n ->
-        frequency
-          [1, map leaf nat;
-           2, map2 node (self (n/2)) (self (n/2))]
-      ))
+        let g = QCheck.Gen.(sized @@ fix
+                              (fun self n -> match n with
+                                 | 0 -> map leaf nat
+                                 | n ->
+                                   frequency
+                                     [1, map leaf nat;
+                                      2, map2 node (self (n/2)) (self (n/2))]
+                              ))
 
-  ]}
+      ]}
 
   *)
 
+  val nat_split2 : int -> (int * int) t
+  (** [nat_split2 n] generates pairs [(n1, n2)] of natural numbers
+      with [n1 + n2 = n].
+
+      This is useful to split sizes to combine sized generators.
+
+      @since 0.18
+  *)
+
+  val pos_split2 : int -> (int * int) t
+  (** [pos_split2 n] generates pairs [(n1, n2)] of strictly positive
+      (nonzero) natural numbers with [n1 + n2 = n].
+
+      @raise Invalid_argument unless [n >= 2].
+
+      This is useful to split sizes to combine sized generators.
+
+      @since 0.18
+  *)
+
+  val nat_split : size:int -> int -> int array t
+  (** [nat_split ~size:k n] generates [k]-sized arrays [n1,n2,..nk]
+      of natural numbers in [[0;n]] with [n1 + n2 + ... + nk = n].
+
+      This is useful to split sizes to combine sized generators.
+
+      Complexity O(k log k).
+
+      @since 0.18
+  *)
+
+  val pos_split : size:int -> int -> int array t
+  (** [pos_split ~size:k n] generates [k]-sized arrays [n1,n2,..nk]
+      of strictly positive (non-zero) natural numbers with
+      [n1 + n2 + ... + nk = n].
+
+      This is useful to split sizes to combine sized generators.
+
+      Complexity O(k log k).
+
+      @raise Invalid_argument unless [0 < k <= n] or [0 = k = n].
+
+      @since 0.18
+  *)
+
+  val delay : (unit -> 'a t) -> 'a t
+  (** Delay execution of some code until the generator is actually called.
+      This can be used to manually implement recursion or control flow
+      in a generator.
+      @since 0.17 *)
+
   val generate : ?rand:Random.State.t -> n:int -> 'a t -> 'a list
   (** [generate ~n g] generates [n] instances of [g]. *)
 
   val generate1 : ?rand:Random.State.t -> 'a t -> 'a
   (** [generate1 g] generates one instance of [g]. *)
 
-  include Qcheck_ops.S with type 'a t_let := 'a t
-  (** @since NEXT_RELEASE *)
+  val ( let+ ) : 'a t -> ('a -> 'b) -> 'b t
+
+  val ( and+ ) : 'a t -> 'b t -> ('a * 'b) t
+
+  val ( let* ) : 'a t -> ('a -> 'b t) -> 'b t
+
+  val ( and* ) : 'a t -> 'b t -> ('a * 'b) t
 end
 
 (** {2 Pretty printing} *)
@@ -507,8 +602,13 @@ module Iter : sig
   val flatten : 'a t t -> 'a t
   (** @since 0.8 *)
 
-  include Qcheck_ops.S with type 'a t_let := 'a t
-  (** @since NEXT_RELEASE *)
+  val ( let+ ) : 'a t -> ('a -> 'b) -> 'b t
+
+  val ( and+ ) : 'a t -> 'b t -> ('a * 'b) t
+
+  val ( let* ) : 'a t -> ('a -> 'b t) -> 'b t
+
+  val ( and* ) : 'a t -> 'b t -> ('a * 'b) t
 end
 
 (** {2 Shrink Values}
@@ -694,6 +794,12 @@ val gen : 'a arbitrary -> 'a Gen.t
 (** Access the underlying random generator of this arbitrary object.
     @since 0.6 *)
 
+val get_gen : 'a arbitrary -> 'a Gen.t
+(** Access the underlying random generator of this arbitrary object.
+    @since 0.6 *)
+
+val get_print : 'a arbitrary -> 'a Print.t option
+
 (** {2 Tests}
 
     A test is a universal property of type [foo -> bool] for some type [foo],
@@ -708,7 +814,7 @@ val gen : 'a arbitrary -> 'a Gen.t
 
 (** Result of running a test *)
 module TestResult : sig
-  type 'a counter_ex = {
+  type 'a counter_ex = 'a QCheck2.TestResult.counter_ex = {
     instance: 'a; (** The counter-example(s) *)
 
     shrink_steps: int; (** How many shrinking steps for this counterex *)
@@ -722,7 +828,7 @@ module TestResult : sig
 
   (** Result state.
       changed in 0.10 (move to inline records, add Fail_other) *)
-  type 'a state =
+  type 'a state = 'a QCheck2.TestResult.state =
     | Success
     | Failed of {
         instances: 'a failed_state; (** Failed instance(s) *)
@@ -735,17 +841,15 @@ module TestResult : sig
       } (** Error, backtrace, and instance that triggered it *)
 
   (* result returned by running a test *)
-  type 'a t = private {
-    mutable state : 'a state;
-    mutable count: int;  (* Number of tests *)
-    mutable count_gen: int; (* Number of generated cases *)
-    collect_tbl: (string, int) Hashtbl.t lazy_t;
-    stats_tbl: ('a stat * (int, int) Hashtbl.t) list; (** @since 0.6 *)
-    mutable warnings: string list;
-    mutable instances: 'a list;
-    (** List of instances used for this test, in no particular order.
-        @since 0.9 *)
-  }
+  type 'a t = 'a QCheck2.TestResult.t
+
+  val get_count : _ t -> int
+  (** Get the count of a cell.
+     @since 0.5.3 *)
+
+  val get_count_gen : _ t -> int
+
+  val get_state : 'a t -> 'a state
 
   val collect : _ t -> (string,int) Hashtbl.t option
   (** Obtain statistics
@@ -764,9 +868,29 @@ module TestResult : sig
       @since 0.9 *)
 end
 
+(** Module related to individual tests.
+     @since 0.18 most of it moved to {!QCheck2},
+     and the type ['a cell] was made a private implementation detail.
+*)
 module Test : sig
-  type 'a cell
-  (** A single property test *)
+  type res = QCheck2.Test.res =
+    | Success
+    | Failure
+    | FalseAssumption
+    | Error of exn * string
+  type 'a event = 'a QCheck2.Test.event =
+    | Generating
+    | Collecting of 'a
+    | Testing of 'a
+    | Shrunk of int * 'a
+    | Shrinking of int * int * 'a
+
+  type 'a cell = 'a QCheck2.Test.cell
+  type 'a handler = 'a QCheck2.Test.handler
+  type 'a step = 'a QCheck2.Test.step
+  type 'a callback = 'a QCheck2.Test.callback
+
+  type t = QCheck2.Test.t
 
   val fail_report : string -> 'a
   (** Fail the test with some additional message that will
@@ -807,23 +931,23 @@ module Test : sig
         (since 0.10)
   *)
 
-  val get_arbitrary : 'a cell -> 'a arbitrary
   val get_law : 'a cell -> ('a -> bool)
+  (** @deprecated use {!QCheck2.Test.get_law} instead *)
   val get_name : _ cell -> string
+  (** @deprecated use {!QCheck2.Test.get_name} instead *)
   val set_name : _ cell -> string -> unit
+  (** @deprecated use {!QCheck2.Test.set_name} instead *)
 
   val get_count : _ cell -> int
   (** Get the count of a cell.
+      @deprecated use {!QCheck2.Test.get_count} instead
       @since 0.5.3 *)
 
   val get_long_factor : _ cell -> int
   (** Get the long factor of a cell.
+      @deprecated use {!QCheck2.Test.get_long_factor} instead
       @since 0.5.3 *)
 
-  type t = Test : 'a cell -> t
-  (** Same as ['a cell], but masking the type parameter. This allows to
-      put tests on different types in the same list of tests. *)
-
   val make :
     ?if_assumptions_fail:([`Fatal | `Warning] * float) ->
     ?count:int -> ?long_factor:int -> ?max_gen:int -> ?max_fail:int ->
@@ -833,94 +957,26 @@ module Test : sig
       See {!make_cell} for a description of the parameters.
   *)
 
-  (** {3 Running the test} *)
+  include module type of QCheck2.Test_exceptions
 
-  exception Test_fail of string * string list
-  (** Exception raised when a test failed, with the list of counter-examples.
-      [Test_fail (name, l)] means test [name] failed on elements of [l]. *)
-
-  exception Test_error of string * string * exn * string
-  (** Exception raised when a test raised an exception [e], with
-      the sample that triggered the exception.
-      [Test_error (name, i, e, st)]
-      means [name] failed on [i] with exception [e], and [st] is the
-      stacktrace (if enabled) or an empty string. *)
-
-  val print_instance : 'a arbitrary -> 'a -> string
-  val print_c_ex : 'a arbitrary -> 'a TestResult.counter_ex -> string
-  val print_fail : 'a arbitrary -> string -> 'a TestResult.counter_ex list -> string
+  val print_instance : 'a cell -> 'a -> string
+  val print_c_ex : 'a cell -> 'a TestResult.counter_ex -> string
+  val print_fail : 'a cell -> string -> 'a TestResult.counter_ex list -> string
   val print_fail_other : string -> msg:string -> string
-  val print_error : ?st:string -> 'a arbitrary -> string -> 'a TestResult.counter_ex * exn -> string
+  val print_error : ?st:string -> 'a cell -> string -> 'a TestResult.counter_ex * exn -> string
   val print_test_fail : string -> string list -> string
   val print_test_error : string -> string -> exn -> string -> string
 
-  val print_collect : (string,int) Hashtbl.t -> string
-  (** Print "collect" results.
-      @since 0.6 *)
-
-  val print_stat : ('a stat * (int,int) Hashtbl.t) -> string
-  (** Print statistics.
-      @since 0.6 *)
-
-  val check_result : 'a cell -> 'a TestResult.t -> unit
-  (** [check_result cell res] checks that [res] is [Ok _], and returns unit.
-      Otherwise, it raises some exception.
-      @raise Test_error if [res = Error _]
-      @raise Test_error if [res = Failed _] *)
-
-  type res =
-    | Success
-    | Failure
-    | FalseAssumption
-    | Error of exn * string
-
-  type 'a event =
-    | Generating
-    | Collecting of 'a
-    | Testing of 'a
-    | Shrunk of int * 'a
-    | Shrinking of int * int * 'a
-
-  type 'a handler = string -> 'a cell -> 'a event -> unit
-  (** Handler executed after each event during testing of an instance. *)
-
-  type 'a step = string -> 'a cell -> 'a -> res -> unit
-  (** Callback executed after each instance of a test has been run.
-      The callback is given the instance tested, and the current results
-      of the test. *)
-
-  type 'a callback = string -> 'a cell -> 'a TestResult.t -> unit
-  (** Callback executed after each test has been run.
-      [f name cell res] means test [cell], named [name], gave [res]. *)
-
   val check_cell :
     ?long:bool -> ?call:'a callback ->
     ?step:'a step -> ?handler:'a handler ->
     ?rand:Random.State.t -> 'a cell -> 'a TestResult.t
-  (** [check_cell ~long ~rand test] generates up to [count] random
-      values of type ['a] using [arbitrary] and the random state [st]. The
-      predicate [law] is called on them and if it returns [false] or raises an
-      exception then we have a counter-example for the [law].
-
-      @param long if [true] then multiply the number of instances to generate
-        by the cell's long_factor.
-      @param call function called on each test case, with the result.
-      @param step function called on each instance of the test case, with the result.
-      @return the result of the test.
-  *)
 
   val check_cell_exn :
     ?long:bool -> ?call:'a callback -> ?step:'a step ->
     ?rand:Random.State.t -> 'a cell -> unit
-  (** Same as {!check_cell} but calls  {!check_result} on the result.
-      @raise Test_error if [res = Error _]
-      @raise Test_error if [res = Failed _] *)
 
   val check_exn : ?long:bool -> ?rand:Random.State.t -> t -> unit
-  (** Checks the property against some test cases, and calls {!check_result},
-      which might raise an exception in case of failure.
-      @raise Test_error if [res = Error _]
-      @raise Test_error if [res = Failed _] *)
 end
 
 (** {2 Sub-tests} *)
@@ -969,7 +1025,7 @@ val find_example_gen :
 
 val choose : 'a arbitrary list -> 'a arbitrary
 (** Choose among the given list of generators. The list must not
-  be empty; if it is Invalid_argument is raised. *)
+    be empty; if it is Invalid_argument is raised. *)
 
 val unit : unit arbitrary
 (** Always generates [()], obviously. *)
@@ -1040,7 +1096,7 @@ val pos_int : int arbitrary
 
 val small_int_corners : unit -> int arbitrary
 (** As [small_int], but each newly created generator starts with
- a list of corner cases before falling back on random generation. *)
+    a list of corner cases before falling back on random generation. *)
 
 val neg_int : int arbitrary
 (** Negative int generator (0 included, see {!Gen.neg_int}).
@@ -1118,15 +1174,15 @@ val quad : 'a arbitrary -> 'b arbitrary -> 'c arbitrary -> 'd arbitrary -> ('a *
 (** Combines four generators into a generator of 4-tuples.
     Order matters for shrinking, see {!Shrink.pair} and the likes *)
 
-val option : 'a arbitrary -> 'a option arbitrary
-(** Choose between returning Some random value, or None. *)
+val option : ?ratio:float -> 'a arbitrary -> 'a option arbitrary
+(** Choose between returning Some random value with optional ratio, or None. *)
 
 val fun1_unsafe : 'a arbitrary -> 'b arbitrary -> ('a -> 'b) arbitrary
 (** Generator of functions of arity 1.
     The functions are always pure and total functions:
     - when given the same argument (as decided by Pervasives.(=)), it returns the same value
     - it never does side effects, like printing or never raise exceptions etc.
-    The functions generated are really printable.
+      The functions generated are really printable.
 
     renamed from {!fun1} since 0.6
 
@@ -1244,31 +1300,35 @@ val fun4 :
 (** @since 0.6 *)
 
 val oneofl : ?print:'a Print.t -> ?collect:('a -> string) ->
-             'a list -> 'a arbitrary
+  'a list -> 'a arbitrary
 (** Pick an element randomly in the list. *)
 
 val oneofa : ?print:'a Print.t -> ?collect:('a -> string) ->
-             'a array -> 'a arbitrary
+  'a array -> 'a arbitrary
 (** Pick an element randomly in the array. *)
 
 val oneof : 'a arbitrary list -> 'a arbitrary
-(** Pick a generator among the list, randomly. *)
+(** Pick a generator among the list, randomly.
+    @deprecated this function is badly specified and will not use shrinkers
+    appropriately. Consider using {!Gen.oneof} and then {!make} to build
+    a well behaved arbitrary instance.
+*)
 
 val always : ?print:'a Print.t -> 'a -> 'a arbitrary
 (** Always return the same element. *)
 
 val frequency : ?print:'a Print.t -> ?small:('a -> int) ->
-                ?shrink:'a Shrink.t -> ?collect:('a -> string) ->
-                (int * 'a arbitrary) list -> 'a arbitrary
+  ?shrink:'a Shrink.t -> ?collect:('a -> string) ->
+  (int * 'a arbitrary) list -> 'a arbitrary
 (** Similar to {!oneof} but with frequencies. *)
 
 val frequencyl : ?print:'a Print.t -> ?small:('a -> int) ->
-                (int * 'a) list -> 'a arbitrary
+  (int * 'a) list -> 'a arbitrary
 (** Same as {!oneofl}, but each element is paired with its frequency in
     the probability distribution (the higher, the more likely). *)
 
 val frequencya : ?print:'a Print.t -> ?small:('a -> int) ->
-                (int * 'a) array -> 'a arbitrary
+  (int * 'a) array -> 'a arbitrary
 (** Same as {!frequencyl}, but with an array. *)
 
 val map : ?rev:('b -> 'a) -> ('a -> 'b) -> 'a arbitrary -> 'b arbitrary
@@ -1281,7 +1341,7 @@ val map : ?rev:('b -> 'a) -> ('a -> 'b) -> 'a arbitrary -> 'b arbitrary
 
 val map_same_type : ('a -> 'a) -> 'a arbitrary -> 'a arbitrary
 (** Specialization of [map] when the transformation preserves the type, which
-   makes shrinker, printer, etc. still relevant. *)
+    makes shrinker, printer, etc. still relevant. *)
 
 val map_keep_input :
   ?print:'b Print.t -> ?small:('b -> int) ->
