@@ -120,30 +120,29 @@ module Raw = struct
      @param verbose if true, print statistics and details
      @param print_res if true, print the result on [out] *)
   let callback ~colors ~verbose ~print_res ~print name cell result =
-    let module R = QCheck.TestResult in
-    let module T = QCheck.Test in
-    let arb = T.get_arbitrary cell in
+    let module R = QCheck2.TestResult in
+    let module T = QCheck2.Test in
     let reset_line = if colors then Color.reset_line else "\n" in
     if verbose then (
       print.info "%slaw %s: %d relevant cases (%d total)\n"
-        reset_line name result.R.count result.R.count_gen;
-      begin match QCheck.TestResult.collect result with
+        reset_line name (R.get_count result) (R.get_count_gen result);
+      begin match QCheck2.TestResult.collect result with
         | None -> ()
         | Some tbl ->
-          print_string (QCheck.Test.print_collect tbl)
+          print_string (QCheck2.Test.print_collect tbl)
       end;
     );
     if print_res then (
       (* even if [not verbose], print errors *)
-      match result.R.state with
+      match R.get_state result with
         | R.Success -> ()
         | R.Failed {instances=l} ->
-          print.fail "%s%s\n" reset_line (T.print_fail arb name l);
+          print.fail "%s%s\n" reset_line (T.print_fail cell name l);
         | R.Failed_other {msg} ->
           print.fail "%s%s\n" reset_line (T.print_fail_other name ~msg);
         | R.Error {instance; exn; backtrace} ->
           print.err "%s%s\n" reset_line
-            (T.print_error ~st:backtrace arb name (instance,exn));
+            (T.print_error ~st:backtrace cell name (instance,exn));
     )
 
   let print_std = { info = Printf.printf; fail = Printf.printf; err = Printf.printf }
@@ -197,10 +196,10 @@ type counter = {
 }
 
 type res =
-  | Res : 'a QCheck.Test.cell * 'a QCheck.TestResult.t -> res
+  | Res : 'a QCheck2.Test.cell * 'a QCheck2.TestResult.t -> res
 
 type handler = {
-  handler : 'a. 'a QCheck.Test.handler;
+  handler : 'a. 'a QCheck2.Test.handler;
 }
 
 type handler_gen =
@@ -216,22 +215,16 @@ let pp_counter ~size out c =
     size c.passed size c.expected t
 
 let debug_shrinking_counter_example cell out x =
-  match (QCheck.Test.get_arbitrary cell).QCheck.print with
+  match QCheck2.Test.get_print_opt cell  with
   | None -> Printf.fprintf out "<no printer provided>"
   | Some print -> Printf.fprintf out "%s" (print x)
 
-let debug_shrinking_size cell out x =
-  match (QCheck.Test.get_arbitrary cell).QCheck.small with
-  | None -> ()
-  | Some f -> Printf.fprintf out ", size %d" (f x)
-
 let debug_shrinking_choices_aux ~colors out name i cell x =
   Printf.fprintf out "\n~~~ %a %s\n\n"
     (Color.pp_str_c ~colors `Cyan) "Shrink" (String.make 69 '~');
   Printf.fprintf out
-    "Test %s sucessfully shrunk counter example (step %d%a) to:\n\n%a\n%!"
+    "Test %s successfully shrunk counter example (step %d) to:\n\n%a\n%!"
     name i
-    (debug_shrinking_size cell) x
     (debug_shrinking_counter_example cell) x
 
 let debug_shrinking_choices
@@ -253,17 +246,17 @@ let default_handler
   ~size ~out ~verbose c =
   let handler name cell r =
     let st = function
-      | QCheck.Test.Generating    -> "generating"
-      | QCheck.Test.Collecting _  -> "collecting"
-      | QCheck.Test.Testing _     -> "   testing"
-      | QCheck.Test.Shrunk (i, _) ->
+      | QCheck2.Test.Generating    -> "generating"
+      | QCheck2.Test.Collecting _  -> "collecting"
+      | QCheck2.Test.Testing _     -> "   testing"
+      | QCheck2.Test.Shrunk (i, _) ->
         Printf.sprintf "shrinking: %4d" i
-      | QCheck.Test.Shrinking (i, j, _) ->
+      | QCheck2.Test.Shrinking (i, j, _) ->
         Printf.sprintf "shrinking: %4d.%04d" i j
     in
     (* debug shrinking choices *)
     begin match r with
-      | QCheck.Test.Shrunk (i, x) ->
+      | QCheck2.Test.Shrunk (i, x) ->
           debug_shrinking_choices
           ~colors ~debug_shrink ~debug_shrink_list name cell i x
       | _ ->
@@ -282,10 +275,10 @@ let default_handler
 
 let step ~colors ~size ~out ~verbose c name _ _ r =
   let aux = function
-    | QCheck.Test.Success -> c.passed <- c.passed + 1
-    | QCheck.Test.Failure -> c.failed <- c.failed + 1
-    | QCheck.Test.FalseAssumption -> ()
-    | QCheck.Test.Error _ -> c.errored <- c.errored + 1
+    | QCheck2.Test.Success -> c.passed <- c.passed + 1
+    | QCheck2.Test.Failure -> c.failed <- c.failed + 1
+    | QCheck2.Test.FalseAssumption -> ()
+    | QCheck2.Test.Error _ -> c.errored <- c.errored + 1
   in
   c.gen <- c.gen + 1;
   aux r;
@@ -296,8 +289,8 @@ let step ~colors ~size ~out ~verbose c name _ _ r =
       (if colors then Color.reset_line else "\n") (pp_counter ~size) c name
   )
 
-let callback ~size ~out ~verbose ~colors c name _ _ =
-  let pass = c.failed = 0 && c.errored = 0 in
+let callback ~size ~out ~verbose ~colors c name _ r =
+  let pass = QCheck2.TestResult.is_success r in
   let color = if pass then `Green else `Red in
   if verbose then (
     Printf.fprintf out "%s[%a] %a %s\n%!"
@@ -306,14 +299,14 @@ let callback ~size ~out ~verbose ~colors c name _ _ =
       (pp_counter ~size) c name
   )
 
-let print_inst arb x =
-  match arb.QCheck.print with
+let print_inst cell x =
+  match QCheck2.Test.get_print_opt cell with
   | Some f -> f x
   | None -> "<no printer>"
 
 let expect long cell =
-  let count = QCheck.Test.get_count cell in
-  if long then QCheck.Test.get_long_factor cell * count else count
+  let count = QCheck2.Test.get_count cell in
+  if long then QCheck2.Test.get_long_factor cell * count else count
 
 let expect_size long cell =
   let rec aux n = if n < 10 then 1 else 1 + (aux (n / 10)) in
@@ -325,56 +318,56 @@ let print_messages ~colors out cell l =
     Printf.fprintf out
       "\n+++ %a %s\n\nMessages for test %s:\n\n%!"
       (Color.pp_str_c ~colors `Blue) "Messages"
-      (String.make 68 '+') (QCheck.Test.get_name cell);
+      (String.make 68 '+') (QCheck2.Test.get_name cell);
     List.iter (Printf.fprintf out "%s\n%!") l
   )
 
 let print_success ~colors out cell r =
-  begin match QCheck.TestResult.collect r with
+  begin match QCheck2.TestResult.collect r with
     | None -> ()
     | Some tbl ->
       Printf.fprintf out
         "\n+++ %a %s\n\nCollect results for test %s:\n\n%s%!"
         (Color.pp_str_c ~colors `Blue) "Collect"
-        (String.make 68 '+') (QCheck.Test.get_name cell) (QCheck.Test.print_collect tbl)
+        (String.make 68 '+') (QCheck2.Test.get_name cell) (QCheck2.Test.print_collect tbl)
   end;
   List.iter (fun msg ->
        Printf.fprintf out
          "\n!!! %a %s\n\nWarning for test %s:\n\n%s%!"
         (Color.pp_str_c ~colors `Yellow) "Warning" (String.make 68 '!')
-        (QCheck.Test.get_name cell) msg)
-    (QCheck.TestResult.warnings r);
+        (QCheck2.Test.get_name cell) msg)
+    (QCheck2.TestResult.warnings r);
 
-  if QCheck.TestResult.stats r <> []  then
+  if QCheck2.TestResult.stats r <> []  then
      Printf.fprintf out
        "\n+++ %a %s\n%!"
-       (Color.pp_str_c ~colors `Blue) ("Stats for " ^ QCheck.Test.get_name cell)
+       (Color.pp_str_c ~colors `Blue) ("Stats for " ^ QCheck2.Test.get_name cell)
        (String.make 56 '+');
   List.iter
-    (fun st -> Printf.fprintf out "\n%s%!" (QCheck.Test.print_stat st))
-    (QCheck.TestResult.stats r);
+    (fun st -> Printf.fprintf out "\n%s%!" (QCheck2.Test.print_stat st))
+    (QCheck2.TestResult.stats r);
   ()
 
 let print_fail ~colors out cell c_ex =
   Printf.fprintf out "\n--- %a %s\n\n" (Color.pp_str_c ~colors `Red) "Failure" (String.make 68 '-');
   Printf.fprintf out "Test %s failed (%d shrink steps):\n\n%s\n%!"
-    (QCheck.Test.get_name cell) c_ex.QCheck.TestResult.shrink_steps
-    (print_inst (QCheck.Test.get_arbitrary cell) c_ex.QCheck.TestResult.instance);
-  print_messages ~colors out cell c_ex.QCheck.TestResult.msg_l
+    (QCheck2.Test.get_name cell) c_ex.QCheck2.TestResult.shrink_steps
+    (print_inst cell c_ex.QCheck2.TestResult.instance);
+  print_messages ~colors out cell c_ex.QCheck2.TestResult.msg_l
 
 let print_fail_other ~colors out cell msg =
   Printf.fprintf out "\n--- %a %s\n\n" (Color.pp_str_c ~colors `Red) "Failure" (String.make 68 '-');
-  Printf.fprintf out "Test %s failed:\n\n%s\n%!" (QCheck.Test.get_name cell) msg
+  Printf.fprintf out "Test %s failed:\n\n%s\n%!" (QCheck2.Test.get_name cell) msg
 
 let print_error ~colors out cell c_ex exn bt =
   Printf.fprintf out "\n=== %a %s\n\n" (Color.pp_str_c ~colors `Red) "Error" (String.make 70 '=');
   Printf.fprintf out "Test %s errored on (%d shrink steps):\n\n%s\n\nexception %s\n%s\n%!"
-    (QCheck.Test.get_name cell)
-    c_ex.QCheck.TestResult.shrink_steps
-    (print_inst (QCheck.Test.get_arbitrary cell) c_ex.QCheck.TestResult.instance)
+    (QCheck2.Test.get_name cell)
+    c_ex.QCheck2.TestResult.shrink_steps
+    (print_inst cell c_ex.QCheck2.TestResult.instance)
     (Printexc.to_string exn)
     bt;
-  print_messages ~colors out cell c_ex.QCheck.TestResult.msg_l
+  print_messages ~colors out cell c_ex.QCheck2.TestResult.msg_l
 
 let run_tests
     ?(handler=default_handler)
@@ -382,8 +375,8 @@ let run_tests
     ?(debug_shrink=debug_shrink()) ?(debug_shrink_list=debug_shrink_list())
     ?(out=stdout) ?rand l =
   let rand = match rand with Some x -> x | None -> random_state_ ~colors () in
-  let module T = QCheck.Test in
-  let module R = QCheck.TestResult in
+  let module T = QCheck2.Test in
+  let module R = QCheck2.TestResult in
   let pp_color = Color.pp_str_c ~bold:true ~colors in
   let size = List.fold_left (fun acc (T.Test cell) ->
       max acc (expect_size long cell)) 4 l in
@@ -393,6 +386,7 @@ let run_tests
       (size + 4) "generated" size "error"
       size "fail" size "pass" size "total";
   let aux_map (T.Test cell) =
+    let rand = Random.State.copy rand in
     let expected = expect long cell in
     let start = Unix.gettimeofday () in
     let c = {
@@ -403,7 +397,7 @@ let run_tests
       Printf.fprintf out "%s[ ] %a %s%!"
         (if colors then Color.reset_line else "")
         (pp_counter ~size) c (T.get_name cell);
-    let r = QCheck.Test.check_cell ~long ~rand
+    let r = QCheck2.Test.check_cell ~long ~rand
         ~handler:(handler ~colors ~debug_shrink ~debug_shrink_list
                     ~size ~out ~verbose c).handler
         ~step:(step ~colors ~size ~out ~verbose c)
@@ -414,8 +408,8 @@ let run_tests
   in
   let res = List.map aux_map l in
   let aux_fold (total, fail, error, warns) (Res (cell, r)) =
-    let warns = warns + List.length r.R.warnings in
-    let acc = match r.R.state with
+    let warns = warns + List.length (R.get_warnings r) in
+    let acc = match R.get_state r with
       | R.Success ->
         print_success ~colors out cell r;
         (total + 1, fail, error, warns)
